%This script performs Genetic algorithm optimization with additional local
%optimization by Multistart/fmincon.

warning('off','all')

%% Set up and run GA
rng(0) %set default seed 
NonlinCon=[]; %no nonlinear constraints, for ga 
ConstraintFunction=[]; %set constraint for the problem
dispalg='iter'; %set whether algorithm iterations are displayed.

gen=500; %max number of generation for GA
stgenlim=50; %max number of stall generations (i.e., no improvement found)

initpop=[]; %set initial population (if smaller than popsize, MATLAB will
%randomly draw the rest. If [], the whole population is randomly drawn.
%Can be a row vector of dimension numpar or a matrix. Each row of the
%matrix is then a candidate initial value.

popsize=300; %population size
elcnt=3; %elite count - number of elite individuals retained in population

tolfunga=1e-10; %tolerance level for improvement in the objective for GA
tolconga=1e-10; %tolerance level for constraint for GA

usepga=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []
%In later versions of Matlab, 1 and 0 can also be used respectively. 




gaoptions=gaoptimset; %create options structure with default values
gaoptions = gaoptimset(gaoptions,'Generation',gen,'TolFun',tolfunga,'PopulationSize',popsize,'InitialPopulation', initpop,'UseParallel',usepga);
gaoptions = gaoptimset(gaoptions,'Display',dispalg,'StallGenLimit',stgenlim,'TolCon',tolconga,'PopInitRange',[lb;ub],'EliteCount',elcnt,'NonlinConAlgorithm',NonlinCon);

%Run GA
[xestga,fvalga,exitflagga,outputga,populationga,scoresga]=ga(ObjectiveFunction,numpar,[],[],[],[],lb,ub,ConstraintFunction,gaoptions);

%% Set up and run Multistart
%numrpoints=10; %number of random starting points for Multistart

usepms=['Always']; %Set to 'Always' to use parallel computation, otherwise to 'Never' or []

% settings for fmincon
maxit=1000; % set max number of iterations
maxfev=20000; % set max number of function evaluations
tolfunfmc=1e-10; %tolerance level for improvement in the objective for fmincon
tolconfmc=1e-10; %tolerance level for constraint for fmincon
tolx=1e-10; %tolerance on solution value

localg='active-set'; %set main local algorithm to be used for multistart



%Set fmincon options
optfmc=optimset('fmincon'); %create default option structure for fmincon
optfmc=optimset('MaxIter',maxit,'MaxFunEvals',maxfev,'TolFun',tolfunfmc,'TolCon',tolconfmc,'TolX',tolx,'Algorithm',localg);

msga = MultiStart('UseParallel',usepms,'Display',dispalg); %set multistart structure

%load the prior mean as a starting value
load prm.mat;
xx0=prm(ind);

gapoints0=xx0';
gapoints1=populationga(1:9,:); %take first 10 points of the final population
gapoints2=populationga([10:((popsize-10)/10):popsize]',:); %add 10 equally spaced points from the rest
gapoints=CustomStartPointSet([gapoints0;gapoints1;gapoints2]); %stack together starting points for Multistart
%rpointsga=RandomStartPointSet('NumStartPoints',numrpoints); %preset random points
%spointsga={gapoints,rpointsga};
spointsga=gapoints;

problemga = createOptimProblem('fmincon','x0',xx0',...
'objective',ObjectiveFunction,'lb',lb,'ub',ub,'nonlcon',ConstraintFunction,...
'options',optfmc); %create problem structure

[xestga2,fvalga2,exitflagga2,outputga2,minimaga] = run(msga,problemga,spointsga); %run multistart with 50 extra random points
