function lp = logprm(theta,compo)
%Computes the prior for SW07 model. 

%Some code here is borrowed from An and Schorfheide's (2007) 
%logprior.m file
%Note this is for estimation including both the mean and spectrum

n  = length(theta);
%index priors by 1 - Beta, 2 - Gamma, 3 - Normal, 4 - Invgamma
%priorind=[1,1,1,3,1,3,3,1,3,1,1,1,1,3,3,3,3,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,3,2]'; %SW paper
priorind=[3,1,1,3,1,3,3,1,3,1,1,1,1,3,3,3,3,1,1,1,1,1,1,1,1,4,4,4,4,4,4,4,3,2,2,3]'; %SW dynare
%compo=[1,4:19,21:23,26,28:30,33:36];
priorind=priorind(compo);

%Select the means and the std. dev.'s of the prior distributions, as in
%SW07 dynare code
swparm=[0.5,0.25;0.5,0.2;0.5,0.2;0.3,0.05;0.5,0.15;4,1.5;1.5,0.375;0.7,0.1;1.25,0.125;0.5,0.15;...
    0.5,0.10;0.5,0.15;0.5,0.10;2,0.75;1.5,0.25;0.125,0.05;0.125,0.05;0.75,0.1;0.5,0.2;0.5,0.2;...
    0.5,0.2;0.5,0.2;0.5,0.2;0.5,0.2;0.5,0.2;0.1,2;0.1,2;0.1,2;0.1,2;0.1,2;0.1,2;0.1,2;0.4,0.1;...
    0.25,0.1;0.625,0.1;0,2]; %note 
swparm=swparm(compo,:);

y = 0;
a = 0;
b = 0;
for i=1:n

	shape = priorind(i); %determine pdf shape
	para1 = swparm(i,1);
	para2 = swparm(i,2);

	% BETA prior
	if shape==1
		a = (1-para1)*para1^2/para2^2 - para1;
		b = a*(1/para1 - 1);
		y = y + lnpdfbeta(theta(i),a,b);

	% GAMMA prior
	elseif shape==2
		b = para2^2/para1;
		a = para1/b;
		y = y + lnpdfgam(theta(i),a,b);

	% NORMAL prior
	elseif shape==3
		a = para1;
		b = para2;
		y = y + lnpdfnorm(theta(i),a,b);

	% INVGAMMA prior
	elseif shape==4
		a = para1;
		b = para2;
        [s,nu] = inverse_gamma_specification(a,b,1);
        a=sqrt(s/nu);
        b=nu;
        
		y = y + lnpdfig(theta(i),a,b);

    end
    lp=y; % the log prior density
end



function [s,nu] = inverse_gamma_specification(mu,sigma,type)

% function [s,nu] = inverse_gamma_specification(mu,sigma,type)
% Specification of the inverse Gamma function parameters
% X ~ IG(s,nu)
%
% INPUTS
%    mu:      expectation
%    sigma:   standard deviation 
%    type=1:  inverse Gamma 1 
%    type=2:  inverse Gamma 2 

% OUTPUTS
%    s:       shape parameter
%    nu:      scale parameter 
%        
% SPECIAL REQUIREMENTS
%    none

% Copyright (C) 2003-2008 Dynare Team

sigma2 = sigma^2;
mu2 = mu^2;

if type == 2       % Inverse Gamma 2   
    nu   = 2*(2+mu2/sigma2);
    s    = 2*mu*(1+mu2/sigma2);
elseif type == 1   % Inverse Gamma 1 
    if sigma2 < Inf
        nu = sqrt(2*(2+mu2/sigma2));
        nu2 = 2*nu;
        nu1 = 2;
        err = 2*mu2*gamma(nu/2)^2-(sigma2+mu2)*(nu-2)*gamma((nu-1)/2)^2;
        while abs(nu2-nu1) > 1e-12
            if err > 0
                nu1 = nu;
                if nu < nu2
                    nu = nu2;
                else
                    nu = 2*nu;
                    nu2 = nu;
                end
            else
                nu2 = nu;
            end
            nu =  (nu1+nu2)/2;
            err = 2*mu2*gamma(nu/2)^2-(sigma2+mu2)*(nu-2)*gamma((nu-1)/2)^2;
        end
        s = (sigma2+mu2)*(nu-2);
    else
        nu  = 2;
        s   = 2*mu2/pi;
    end   
else
    s  = -1;
    nu = -1;
end



%
% logs of density functions
%
%These functions borrowed from AS(2007)
% beta
function y=lnpdfbeta(x,a,b)
% LNPDFBETA(X,A,B)
%	calculates log BETA(A,B) at X

% Sungbae An
% 03/03/2002
y = gammaln(a+b) - gammaln(a) - gammaln(b) + (a-1)*log(x) + (b-1)*log(1-x);

% gamma
function y=lnpdfgam(x,a,b)
% LNPDFGAM(X,A,B)
%	calculates log GAMMA(A,B) at X

% 03/03/2002
% Sungbae An
y = -gammaln(a) -a*log(b)+ (a-1)*log(x) -x/b ;

% inverse gamma
function y = lnpdfig(x,a,b)
% LNPDFIG(X,A,B)
%	calculates log INVGAMMA(A,B) at X

% 03/03/2002
% Sungbae An
y = log(2) - gammaln(b/2) + (b/2)*log(b*a^2/2) - ( (b+1)/2 )*log(x^2) - b*a^2/(2*x^2);


% normal
function y = lnpdfnorm(x,a,b)
% LNPDFNORM(X,A,B)
%	calculates log NORMAL(A,B) at X

% 03/03/2002
% Sungbae An
y = -0.5*log(2*pi) - log(b) - 0.5*(x-a)^2/b^2;