%Computes the smallest eigenvalue of the G matrix.
%For the five shocks model.
%spec, n, int, derv and dif_para below can be varied to examine the result sensitivity
%% The total run time is 5-38 minutes using a singe core depending on the choice of methods, most importantly n.

clear;
spec=2; %set to 0,1 and 2 for the default, first and second alternative specification
n=10000; %the number of evaluation points when computing the integral
int=1;  %set to 1 for Gaussian quadrature and 0 for Riemann sum
derv=2; %set to 1,2 and 3 to use two-point method, symmetric quotient and five-point method to compute the derivative
dif_para=1e-7; %differentiation step size 


%You shouldn't have to change the code below
timega=tic;%time keeping
if spec==0
    load parasim5;
    theta=mean(parasim5);
elseif spec==1
    load parasim5s1;
    theta=mean(parasim5s1);
elseif spec==2
    load parasim5s2;
    theta=mean(parasim5s2);
end

dimy=7;
ci=-1i; %imaginary -i
pi2=2*pi; 





%[x,wg] = mp.GaussLegendre(n,-pi1,pi1);
%% Quadrature abscissae and weights
if int==1
     %[x,wg] = mp.GaussLegendre(n,-pi1,pi1);
     [x,wg] = lgwt(n,-pi,pi); 
else
    % Frequencies to evaluate the integral using the Riemann sum
    x=pi2*(-(n/2-1):1:n/2)'/n;
    wg=ones(n,1);
    display('Riemann sum');
   
end

  M=eye(dimy);
  M(2,2)=0;
  %M(6,6)=0;
  M(7,7)=0;
      

theta0=[theta(1),0,theta(2:18),0,theta(19:22),0,theta(23),0,theta(24:27),0,theta(28:31)];
sp_mat=Fmat(theta0,M,n,x);



kk=length(theta)-4;


%% Numerical derivative
nsp_mat=zeros(dimy*n,dimy*kk); %blank for derivatives (a-d)
psp_mat=nsp_mat; %blank for derivatives (a+d)
Dif_mat=nsp_mat;
n2sp_mat=zeros(dimy*n,dimy*kk); %blank for derivatives (a-d)
p2sp_mat=nsp_mat;

for j=1:kk;

%perturb one parameter at a time
ntheta=theta;
ptheta=theta;
n2theta=theta;
p2theta=theta;
ntheta(j)=theta(j)-dif_para;
ptheta(j)=theta(j)+dif_para;
n2theta(j)=theta(j)-2*dif_para;
p2theta(j)=theta(j)+2*dif_para;
npara=[ntheta(1),0,ntheta(2:18),0,ntheta(19:22),0,ntheta(23),0,ntheta(24:27),0,ntheta(28:31)];
ppara=[ptheta(1),0,ptheta(2:18),0,ptheta(19:22),0,ptheta(23),0,ptheta(24:27),0,ptheta(28:31)];

n2para=[n2theta(1),0,n2theta(2:18),0,n2theta(19:22),0,n2theta(23),0,n2theta(24:27),0,n2theta(28:31)];
p2para=[p2theta(1),0,p2theta(2:18),0,p2theta(19:22),0,p2theta(23),0,p2theta(24:27),0,p2theta(28:31)];


nsp_mat(:,((j-1)*dimy+1):(j*dimy))=Fmat(npara,M,n,x); %spectra at perturbed theta0
psp_mat(:,((j-1)*dimy+1):(j*dimy))=Fmat(ppara,M,n,x); %spectra at perturbed theta0

n2sp_mat(:,((j-1)*dimy+1):(j*dimy))=Fmat(n2para,M,n,x); %spectra at perturbed theta0
p2sp_mat(:,((j-1)*dimy+1):(j*dimy))=Fmat(p2para,M,n,x); %spectra at perturbed theta0

end


if derv==3

for j=1:kk
dif2=n2sp_mat(:,((j-1)*dimy+1):(j*dimy))-p2sp_mat(:,((j-1)*dimy+1):(j*dimy));
dif1=psp_mat(:,((j-1)*dimy+1):(j*dimy))-nsp_mat(:,((j-1)*dimy+1):(j*dimy));
Dif_mat(:,((j-1)*dimy+1):(j*dimy))=(dif2+8*dif1)/(12*dif_para);

end

end

if derv==2

for j=1:kk

Dif_mat(:,((j-1)*dimy+1):(j*dimy))=(psp_mat(:,((j-1)*dimy+1):(j*dimy))-nsp_mat(:,((j-1)*dimy+1):(j*dimy)))/(2*dif_para);

end

end


if derv==1

for j=1:kk

Dif_mat(:,((j-1)*dimy+1):(j*dimy))=(psp_mat(:,((j-1)*dimy+1):(j*dimy))-sp_mat)/dif_para;

end

end

Gmat=zeros(kk,kk); %blank
for freq=1:n
    
for j=1:kk

for k=1:kk
        aj=Dif_mat(((freq-1)*dimy+1):(freq*dimy),((j-1)*dimy+1):(j*dimy));
        ak=Dif_mat(((freq-1)*dimy+1):(freq*dimy),((k-1)*dimy+1):(k*dimy));
        
        if spec==0
            for i=1:3
                if i==1
                    seli=[2,3,4,5,7];
                elseif i==2
                    seli=1;
                else
                seli=6;
                end
            ajs=aj(seli,seli);
            aks=ak(seli,seli);
            Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(ajs*aks));
            end
        end
        
      if spec==1
        for i=1:2
            if i==1
                seli=[2,3,4,5,7];
            elseif i==2
                seli=[1,6];
            end
        ajs=aj(seli,seli);
        aks=ak(seli,seli);
        Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(ajs*aks));
        end
      end
   
     
     if spec==2
        for i=1:7
        seli=i;
        ajs=aj(seli,seli);
        aks=ak(seli,seli);
        Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(ajs*aks));
        end
    end
        
end
        
end

end


if int==1
    G=real(Gmat);
else
    G=real(pi2*Gmat/n); 
end



    

val=eig(G);
seig=min(abs(val));
s = svd(G);
%s=double(s);
tolmp = max(size(G))*eps(max(s));
[v,d]=eig(G);

display ('-- The smallest eigenvalue is--');
display (min(diag(d)));
display ('-- The tolerance level is--');
display (tolmp);

 timelga=toc(timega); %time taken
