%Computes the smallest eigenvalue of the G matrix with multiple precision
%For the six shocks model.
% spec, n, int, derv and dif_para below can be varied to to examine the result sensitivity
%% The total run time is 2.0-3.4 hours using a singe core depending on the choice of methods, for n=1000.

clear;
mp.Digits(34); %34 digit precision
spec=mp('2');%set to 0,1 and 2 for the default, first and second alternative specification
n=mp('1000');%the number of evaluation points when computing the integral; recommending 1000.
int=mp('1');%set to 1 for Gaussian quadrature and 0 for Riemann sum
derv=mp('2');%set to 1 and 2 to use two-point method or symmetric quotient to compute the derivative
dif_para=mp('1e-10'); %differentiation step size

%You shouldn't have to change the code below
timega=tic;%time keeping
if spec==0
    load parasim6;
    theta=mean(parasim6);
elseif spec==1
    load parasim6;
    theta=mean(parasim6);
elseif spec==2
    load parasim6s2;
    theta=mean(parasim6s2);
end
    
theta=mp(theta);
zerop=mp('0');
dimy=mp('7');
%kk=mp('4');
pi1=mp('pi');
ci=mp('-1i'); %imaginary -i
pi2=mp('2*pi'); %2*pi in mp
onep=mp('1'); %1 in mp
twop=mp('2'); %2 in mp

%[x,wg] = mp.GaussLegendre(n,-pi1,pi1);
%% Quadrature abscissae and weights
if int==1
     [x,wg] = mp.GaussLegendre(n,-pi1,pi1);
else
    % Frequencies to evaluate the integral using the Riemann sum
    x=pi2*(-(n/twop-onep):onep:(n/twop))'/n;
    wg=mp(ones(n,1));
    display('riemann sum');
end

  M=mp(eye(dimy));
  M(2,2)=zerop;
  %M(6,6)=zerop;
  %M(7,7)=zerop;
      

theta0=[theta(1:19),zerop,theta(20:25),zerop,theta(26:34)];
sp_mat=Fmat_mp(theta0,M,n,x);


kk=mp(length(theta)-4);


%% Numerical derivative
nsp_mat=mp(zeros(dimy*n,dimy*kk)); %blank for derivatives (a-d)
psp_mat=nsp_mat; %blank for derivatives (a+d)
Dif_mat=nsp_mat;

for j=onep:kk;

%perturb one parameter at a time
ntheta=theta;
ptheta=theta;
ntheta(j)=theta(j)-dif_para;
ptheta(j)=theta(j)+dif_para;
npara=[ntheta(1:19),zerop,ntheta(20:25),zerop,ntheta(26:34)];
ppara=[ptheta(1:19),zerop,ptheta(20:25),zerop,ptheta(26:34)];


nsp_mat(:,((j-1)*dimy+1):(j*dimy))=Fmat_mp(npara,M,n,x); %spectra at perturbed theta0
psp_mat(:,((j-1)*dimy+1):(j*dimy))=Fmat_mp(ppara,M,n,x); %spectra at perturbed theta0

end


if derv==2

for j=1:kk

Dif_mat(:,((j-1)*dimy+1):(j*dimy))=(psp_mat(:,((j-1)*dimy+1):(j*dimy))-nsp_mat(:,((j-1)*dimy+1):(j*dimy)))/(twop*dif_para);

end

end


if derv==1

for j=1:kk

Dif_mat(:,((j-1)*dimy+1):(j*dimy))=(psp_mat(:,((j-1)*dimy+1):(j*dimy))-sp_mat)/(onep*dif_para);

end

end

Gmat=mp(zeros(kk,kk)); %blank
for freq=1:n
    
for j=1:kk

for k=1:kk
        aj=Dif_mat(((freq-1)*dimy+1):(freq*dimy),((j-1)*dimy+1):(j*dimy));
        ak=Dif_mat(((freq-1)*dimy+1):(freq*dimy),((k-1)*dimy+1):(k*dimy));
        
        if spec==0
        for i=1:2
        if i==1
         seli=[2,3,4,5,6,7];
        elseif i==2
         seli=1;
        
        end
        ajs=aj(seli,seli);
        aks=ak(seli,seli);
       Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(ajs*aks));
        end
        end
        
        if spec==1
        for i=1:2
        if i==1
         seli=[2,3,4,5,6,7];
        elseif i==2
         seli=1;
        end
        ajs=aj(seli,seli);
        aks=ak(seli,seli);
       Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(ajs*aks));
        end
        end
        
        if spec==2
        for i=1:7
        seli=i;
        ajs=aj(seli,seli);
        aks=ak(seli,seli);
        Gmat(j,k)=Gmat(j,k)+wg(freq)*sum(diag(ajs*aks));
        end
        end
        
        
        
end
        
end

end


if int==1
    Gmp=real(Gmat);
else
    Gmp=real(pi2*Gmat/n); 
end

  

valmp=eig(Gmp);
seigmp=min(abs(valmp));
s = svd(Gmp);
s=double(s);
tolmp = max(size(Gmp))*eps(max(s));
[v,d]=eig(Gmp);

display ('-- The smallest eigenvalue is--');
display (min(diag(d)));
display ('-- The tolerance level is--');
display (tolmp);

timelga=toc(timega); %time taken
