% The program generates values from the quasoi posterior distribution via the random walk Metropolis Hastings Algorithm
% To utilize multiple cores, several independent chains are produced and then merged.

%% The total runtime (for 200000 draws) is about 3.3 hours on a Xeon E5-2665 single-core 2.4Ghz processor, using Matlab R2015a.

clear;
timega=tic;
%% Load parameter bounds 
lb=[0.010;0.010;0.010;0.010;0.010;2;0.250;0.0010;1;0.010;0.30;0.010;0.50;0.250;1.00;0.0010;0.0010;0.50;0.010;0.010;0.010;0.010;0.010;0.010;0.0010;0.010;0.0250;0.010;0.010;0.010;0.010;0.010;0.10;0.010;0.10;-10];
ub=[2;0.9999;0.9999;1;1;15;3;0.99000;3;0.99000;0.95000;0.99000;0.95000;10;3;0.50000;0.50000;0.97500;0.9999;0.9999;0.9999;0.9999;0.9999;0.9999;0.9999;3;5;3;3;3;3;3;0.80000;2;2;10];
lb(13)=0.1; 
compo=[1,4:19,21:23,26,28:30,33:36];  
lb=lb(compo);
ub=ub(compo);
   
%% Specify input parameters
cc=0.1; %scaling
cc0=1;

nsim    = 200000; %total number of draws
nsim=800;
nchain=8; %number of independent chains

npara=28;%number of parameters

display ('Scaling of Hess');
display (cc);

load fhmult4s1;
load postmod4s1;
para=postmod4s1;
SIGSCALE=fhmult4s1;

simj=nsim/nchain;

numcore=8; %specify the number of cores
% 
% %for versions before R2014a
% matlabpool open local numcore
% 
% %for versions R2014a and above
ppool=parpool('local',numcore); %create parallel pool object 

ObjFunction = @post_sw_4s1;

parfor j=1:nchain
     mchain(ObjFunction,para,cc,cc0,SIGSCALE,lb,ub,simj,j);
     %the individual chains are saved automatically within mchain
end

%% Close parallel pool

% %for versions before R2014a
% matlabpool close
% 
% %for versions R2014a and above
delete(ppool) %delete the parallel pool object 

%Now load individual files, merge them and keep only the merged file
load parasim1;
pdraws=parasim;
clear parasim;
delete parasim1.mat;


for j=2:nchain
    filename=sprintf('parasim%d.mat',j);
    load (filename);
    pdraws=[pdraws;parasim];
    clear parasim;
    delete (filename);
end 

parasim4s1=pdraws;
save parasim4s1 parasim4s1;
timemc4s1=toc(timega);
save timemc4s1 timemc4s1;