% The program generates draws from the quasoi posterior distribution via the random walk Metropolis Hastings Algorithm
% traslated from the gauss code of Lubik and Schorfheide (2004) @ 02/21/02.

%% The total runtime is about 2.5 hours on a Xeon E5-2665 single-core 2.4Ghz processor, using Matlab R2015a.
% The computational time can be reduced substantially by utilizing multiple cores (i.e., running multiple independent chains).
clear;


%% **  Configure the Metropolis Algorithm

cc0     = 1;
cc      = 0.15;
nsim    = 200000;
npara=9;
%[1/tau rstar kappa psi1 psi2 rhoR rhog rhoz sigR sigg sigz, rhogz, pistar]';
lb=[1E-5 1E-5 1E-5 1.00001  1E-5  1E-5    1E-5    1E-5   1E-5 ]';
ub=[100   100   100  100   0.999 0.999 0.999   10           10 ]';  

%/******************************************************** 
%** Load The Mode
%*/
load postmod;
para=postmod;

display ('Scaling of Hess');
display (cc);

load fhmult;
load hessian;
SIGSCALE=fhmult;
SIGPROPINV=hessian;
SIGPROPdim =npara;


[u , s, v] = svd(SIGPROPINV);
SIGPROPlndet = 0;

%/******************************************************** 
%**              Metropolis Hastings Algorithm
%*/

%/* Initialize PARANEW
%*/
timeh=tic;
cpara=npara;


valid = 0;
while valid == 0
    paranew   = para' + cc0*(SIGSCALE*randn(npara,1));
    parabd_ind1 = paranew(1:cpara)>=lb(1:cpara);
    parabd_ind2 = paranew(1:cpara)<=ub(1:cpara);
    if (min(parabd_ind1) == 1) & (min(parabd_ind2) == 1)
       postnew  =  ls_ps_1(paranew');
    else
       postnew = -1E20;
      
    end
   if postnew > -10000
      valid = 1;
   end

end   


iblock = 1;
for iblock=1:1 %can be generalized to have multipe blocks

   parasim = zeros(nsim,npara);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if iblock == 1
      j = 2;
      postsim(1,1) = postnew;
      proppostsim(1,1) = postnew;
      parasim(1,:) = paranew';
      postold      = postnew;
      paraold      = paranew;     
   else
      j=1;
   end      

   while j <= nsim

    paranew = paraold + cc*(SIGSCALE*randn(npara,1));                                                              
    parabd_ind1 = paranew(1:cpara)>=lb(1:cpara);
    parabd_ind2 = paranew(1:cpara)<=ub(1:cpara);
    if (min(parabd_ind1) == 1) & (min(parabd_ind2) == 1)
       postnew  =  ls_ps_1(paranew');
    else
       postnew = -1E20;
      
    end
     proppostsim(j,1) = postnew;
     

     r = min([1; exp(postnew - postold)]);
     
     if rand(1,1) < r
       %/* Accept proposed jump
       %*/
       postsim(j,1) = postnew;
       parasim(j,:) = paranew';
       paraold = paranew;
       postold = postnew;
 
     else
       %/* Reject proposed jump
       %*/     
       postsim(j,1) = postold;
       parasim(j,:) = paraold';
       rej(j) = 1;
     end
     j=j+1;
   end

   display ('Rejection: r / rej / perct');
   display (sum(rej)/nsim);
   
   save parasim parasim;
end 
timel=toc(timeh);
        
